@@ -133,6 +133,8 @@ struct _MetaWaylandDmaBufBuffer
 {
   GObject parent;
 
+  MetaWaylandDmaBufManager *manager;
+
   int width;
   int height;
   uint32_t drm_format;
@@ -783,7 +785,8 @@ buffer_params_create_common (struct wl_client   *client,
     wl_resource_create (client, &wl_buffer_interface, 1, buffer_id);
   wl_resource_set_implementation (buffer_resource, &dma_buf_buffer_impl,
                                   dma_buf, NULL);
-  buffer = meta_wayland_buffer_from_resource (buffer_resource);
+  buffer = meta_wayland_buffer_from_resource (dma_buf->manager->compositor,
+                                              buffer_resource);
 
   meta_wayland_buffer_realize (buffer);
   if (!meta_wayland_dma_buf_realize_texture (buffer, &error))
@@ -857,10 +860,13 @@ dma_buf_handle_create_buffer_params (struct wl_client   *client,
                                      struct wl_resource *dma_buf_resource,
                                      uint32_t            params_id)
 {
+  MetaWaylandDmaBufManager *dma_buf_manager =
+    wl_resource_get_user_data (dma_buf_resource);
   struct wl_resource *params_resource;
   MetaWaylandDmaBufBuffer *dma_buf;
 
   dma_buf = g_object_new (META_TYPE_WAYLAND_DMA_BUF_BUFFER, NULL);
+  dma_buf->manager = dma_buf_manager;
 
   params_resource =
     wl_resource_create (client,
@@ -1282,7 +1288,7 @@ add_format (MetaWaylandDmaBufManager *dma_buf_manager,
                                          &num_modifiers, &error))
     {
       g_warning ("Failed to query modifiers for format 0x%" PRIu32 ": %s",
-                 drm_format, error ? error->message : "unknown error");
+                 drm_format, error->message);
       goto add_fallback;
     }
 
@@ -1347,28 +1353,72 @@ init_format_table (MetaWaylandDmaBufManager *dma_buf_manager)
     meta_anonymous_file_new (size, (uint8_t *) format_table);
 }
 
-static void
-init_formats (MetaWaylandDmaBufManager *dma_buf_manager,
-              EGLDisplay                egl_display)
+static EGLint supported_formats[] = {
+  DRM_FORMAT_ARGB8888,
+  DRM_FORMAT_ABGR8888,
+  DRM_FORMAT_XRGB8888,
+  DRM_FORMAT_XBGR8888,
+  DRM_FORMAT_ARGB2101010,
+  DRM_FORMAT_ABGR2101010,
+  DRM_FORMAT_XRGB2101010,
+  DRM_FORMAT_XBGR2101010,
+  DRM_FORMAT_RGB565,
+  DRM_FORMAT_ABGR16161616F,
+  DRM_FORMAT_XBGR16161616F,
+  DRM_FORMAT_XRGB16161616F,
+  DRM_FORMAT_ARGB16161616F
+};
+
+static gboolean
+init_formats (MetaWaylandDmaBufManager  *dma_buf_manager,
+              EGLDisplay                 egl_display,
+              GError                   **error)
 {
+  MetaContext *context = dma_buf_manager->compositor->context;
+  MetaBackend *backend = meta_context_get_backend (context);
+  MetaEgl *egl = meta_backend_get_egl (backend);
+  EGLint num_formats;
+  g_autofree EGLint *driver_formats = NULL;
+  int i, j;
+
   dma_buf_manager->formats = g_array_new (FALSE, FALSE,
                                           sizeof (MetaWaylandDmaBufFormat));
 
-  add_format (dma_buf_manager, egl_display, DRM_FORMAT_ARGB8888);
-  add_format (dma_buf_manager, egl_display, DRM_FORMAT_ABGR8888);
-  add_format (dma_buf_manager, egl_display, DRM_FORMAT_XRGB8888);
-  add_format (dma_buf_manager, egl_display, DRM_FORMAT_XBGR8888);
-  add_format (dma_buf_manager, egl_display, DRM_FORMAT_ARGB2101010);
-  add_format (dma_buf_manager, egl_display, DRM_FORMAT_ABGR2101010);
-  add_format (dma_buf_manager, egl_display, DRM_FORMAT_XRGB2101010);
-  add_format (dma_buf_manager, egl_display, DRM_FORMAT_XBGR2101010);
-  add_format (dma_buf_manager, egl_display, DRM_FORMAT_RGB565);
-  add_format (dma_buf_manager, egl_display, DRM_FORMAT_ABGR16161616F);
-  add_format (dma_buf_manager, egl_display, DRM_FORMAT_XBGR16161616F);
-  add_format (dma_buf_manager, egl_display, DRM_FORMAT_XRGB16161616F);
-  add_format (dma_buf_manager, egl_display, DRM_FORMAT_ARGB16161616F);
+  if (!meta_egl_query_dma_buf_formats (egl, egl_display, 0, NULL, &num_formats,
+                                       error))
+    return FALSE;
+
+  if (num_formats == 0)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "EGL doesn't support any DRM formats");
+      return FALSE;
+    }
+
+  driver_formats = g_new0 (EGLint, num_formats);
+  if (!meta_egl_query_dma_buf_formats (egl, egl_display, num_formats,
+                                       driver_formats, &num_formats, error))
+    return FALSE;
+
+  for (i = 0; i < G_N_ELEMENTS (supported_formats); i++)
+    {
+      for (j = 0; j < num_formats; j++)
+        {
+          if (supported_formats[i] == driver_formats[j])
+            add_format (dma_buf_manager, egl_display, supported_formats[i]);
+        }
+    }
+
+  if (dma_buf_manager->formats->len == 0)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "EGL doesn't support any DRM formats supported by the "
+                   "compositor");
+      return FALSE;
+    }
 
   init_format_table (dma_buf_manager);
+  return TRUE;
 }
 
 static void
@@ -1495,6 +1545,16 @@ initialize:
 
   dma_buf_manager = g_object_new (META_TYPE_WAYLAND_DMA_BUF_MANAGER, NULL);
 
+  dma_buf_manager->compositor = compositor;
+  dma_buf_manager->main_device_id = device_id;
+
+  if (!init_formats (dma_buf_manager, egl_display, &local_error))
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "No supported formats detected: %s", local_error->message);
+      return NULL;
+    }
+
   if (!wl_global_create (compositor->wayland_display,
                          &zwp_linux_dmabuf_v1_interface,
                          protocol_version,
@@ -1506,10 +1566,6 @@ initialize:
       return NULL;
     }
 
-  dma_buf_manager->compositor = compositor;
-  dma_buf_manager->main_device_id = device_id;
-
-  init_formats (dma_buf_manager, egl_display);
   init_default_feedback (dma_buf_manager);
 
   return g_steal_pointer (&dma_buf_manager);
