@@ -36,6 +36,9 @@ MetaWaylandCompositor * meta_wayland_compositor_new             (MetaContext *co
 
 void                    meta_wayland_compositor_prepare_shutdown (MetaWaylandCompositor *compositor);
 
+void                    meta_wayland_compositor_init_display    (MetaWaylandCompositor *compositor,
+                                                                 MetaDisplay           *display);
+
 META_EXPORT_TEST
 MetaWaylandCompositor  *meta_wayland_compositor_get_default     (void);
 
@@ -91,8 +94,5 @@ void                    meta_wayland_compositor_notify_surface_id (MetaWaylandCo
                                                                    int                    id,
                                                                    MetaWaylandSurface    *surface);
 
-META_EXPORT_TEST
-MetaXWaylandManager *   meta_wayland_compositor_get_xwayland_manager (MetaWaylandCompositor *compositor);
-
 #endif
 
